"""
SNOWBAL-module v1.2
Script to determine the approximate best representative wavelength with water vapour, liquid water content and/or ice water content.
Output: representative wavelengths for clear-sky direct, clear-sky diffuse and cloudy diffuse conditions in text files. 

MADE BY C.T. van Dalum et al. (2018, 2019)
"""

#### Packages ####

import tartes
import os.path
from impurities import Soot
from impurities import HULIS
import numpy as np
from shutil import copyfile

###
# INPUT
###   

print ("Reading the input")

savedir = "/Results/snowbal1_2/"                                                                    # Save directory
libradtran_base = '/libRadtran-2.0.2/Own_Files/snowbal1_2/'                                         # Base directory of libradtran data.
basename = "rad_sza="                                                                               # Base name of the input and output files, beyond "=" is the angle to be considered
libradtrantemplate = "rad_per_sza_template_clouds.INP"                                              # Libradtran input template name
alb_file_location = "/Albedo_profiles_for_DISORT/snowbal1_2/"                                       # Location of the albedo file that is used for DISORT
alb_file_name = "albedo_TARTES_sza"                                                                 # Base name of the albedo file

iwplist = [0.0, 0.01, 0.05, 0.1, 0.15, 0.2, 0.5, 1.0, 2.0 ,5.0]                                     # IWP, or vertically integrated liquid water content
lwplist = [0.0,0.05,0.1,0.15,0.2,0.5,1.0,3.0,5.0,10.0,20.0,40.0]                                    # LWP, or vertically integrated liquid water content
PWlist = [0.5,2.0,4.0,6.0,8.0,10.0,13.0,18.0,25.0,40.0]                                             # Precipitable water in kg/m2 to consider for direct radiation

sza = np.linspace(0,53./9.*6. + 53.,16)                                                             # SZA
cloudyangle = 53.0                                                                                  # Angle to use to compute the representative wavelength for cloudy conditions.
  
spectralintervalsnm = np.array([200, 263, 345, 442, 625, 778, 1242, 
                                1299, 1626, 1942, 2151, 2500, 3077],dtype=float)                    # Spectral intervals to consider (so in this case 12 spectral intervals)

# Snow profile         - The attributes of the reference snow layers
ssa = [40,15,10,3]                                                                                  # Specific surface area
density = [200,300,350,450]                                                                         # Snow density
thickness = [0.2,0.5,1,3]                                                                           # Layer thickness

# Impurities
impurity_type = ""                                                                                  # No impurities choose "", for soot: Soot, for hulis: HULIS   
impurities = 0.0 * 10**-9                                                                           # Impurity concentration in g/g

### Switches
makingfiles = 1                                                                                     # 1 if libradtran input files making is on, 0 if off.
runninglibradtran = 1                                                                               # 1 if libradtran is run with inputfiles produced with makingfiles, 0 if off.
verbose = 0                                                                                         # 1 if extra disort output file is produced

#%%   ## Making arrays compatible
for i in range(0,len(sza)):
    if i == 1:
        sza[i] =  "%.11f" % sza[i]
    else:
        sza[i] =  "%.10f" % sza[i]
          
if not os.path.exists(savedir):                                                                     # Making the directory if it does not exist
    os.makedirs(savedir)  
    
sza = np.atleast_1d(sza)
lwplist2 = []
iwplist2 = []
for i in range(0,len(lwplist)):
    lwplist2.append(np.str(lwplist[i]).replace(".",""))
for i in range(0,len(iwplist)):
    iwplist2.append(np.str(iwplist[i]).replace(".",""))
    
#%%

#### MAKING THE INPUT FILES FOR LIBRADTRAN AND RUNNING IT, FOR DIFFERENT SZA VALUES.  
if makingfiles == 1:
    for k in range(0,len(lwplist)):
        for j in range(0,len(iwplist)):
            for m in range(0,2):
                #### m = 0 is for direct radiation, m = 1 is for diffuse radiation
                if lwplist[k] != 0.0 or iwplist[j] != 0.0:
                    if m == 0:
                        continue
                if m == 0:
                    locationfiles = "%s/lwp%s_iwp%s/direct"%(libradtran_base,lwplist2[k],iwplist2[j])
                elif m == 1:
                    locationfiles = "%s/lwp%s_iwp%s/diffuse"%(libradtran_base,lwplist2[k],iwplist2[j])
                print ("Making files: LWP = %s, IWP = %s"%(lwplist[k],iwplist[j]))
        
        
                if not os.path.exists(locationfiles):                               # Making directory if it does not exist
                    os.makedirs(locationfiles)  
            
                copyfile("%s/%s"%(libradtran_base,libradtrantemplate), "%s/%s"%(locationfiles,libradtrantemplate))      # Copy the radiation file into the correct folder
            
                ### MAKING THE CLOUD CONTENT FILES -- LWP
                if lwplist[k] <= 10.:                                               # For thin clouds
                    wcfile = open('%s/WC_small.DAT'%(libradtran_base), 'r+b')
                    wcfile2 = wcfile.read()
                    wcfile.close()
                    wcfile3 = wcfile2.replace('LWC', '%s'%(lwplist[k]/2))           # Divided by the thickness of the cloud in order to get a correct LWP        
                else:                                                               # For thick clouds
                    wcfile = open('%s/WC_large.DAT'%(libradtran_base), 'r+b')
                    wcfile2 = wcfile.read()
                    wcfile.close()
                    wcfile3 = wcfile2.replace('LWC', '%s'%(lwplist[k]/4))  
            
                newfile = open("%s/WC.DAT"%(locationfiles), 'w')
                newfile.write(wcfile3)
                newfile.close()
                
                ### MAKING THE CLOUD CONTENT FILES -- IWP        
                icfile = open('%s/IC.DAT'%(libradtran_base), 'r+b')
                icfile2 = icfile.read()
                icfile.close()                  
                icfile3 = icfile2.replace('IWC', '%s'%(iwplist[j]))  
                newfile2 = open("%s/IC.DAT"%(locationfiles), 'w')
                newfile2.write(icfile3)
                newfile2.close()
                
                for i in range(0,len(sza)):
                    #### Direct radiation
                    if m == 0:
                        for x in range(0,len(PWlist)):         ### Loop over the PW list
                            print ("Making the input for sza = %s degrees and PW = %s, direct"%(sza[i],PWlist[x]))

                            if not os.path.exists('%s/PW=%s'%(locationfiles,PWlist[x])):                               # Making directory if it does not exist
                                os.makedirs('%s/PW=%s'%(locationfiles,PWlist[x]))                              
                            
                            data1 = open('%s/%s'%(libradtran_base,libradtrantemplate), 'r+b')
                            data2 = data1.read()
                            data1.close()
                            data3 = data2.replace('SZA', '%s'%(sza[i]))
                            data4 = data3.replace('ALB', '%s/%s%s.ALB'%(alb_file_location,alb_file_name,sza[np.where(sza == cloudyangle)[0][0]]))
                            data5 = data4.replace('MOLMODIFY', 'mol_modify H2O %s MM'%(PWlist[x]))
                            data6 = data5.replace('WCDATA', '%s/WC.DAT'%(locationfiles))
                            data7 = data6.replace('ICDATA', '%s/IC.DAT'%(locationfiles))
                            if verbose == 1:
                                data7 = data7.replace('quiet', 'verbose')
                        
                            newfile3 = open("%s/PW=%s/%s%s.INP"%(locationfiles,PWlist[x],basename,sza[i]), 'w')
                            if lwplist[k] >= 0.5 or iwplist[j] >= 0.5:      # Use disort instead of sdisort for thick clouds
                                data8 = data7.replace('sdisort', 'disort')
                                newfile3.write(data8)     
                            elif sza[i] == 0.0:                         # Use disort instead of sdisort for sza = 0.0
                                data8= data7.replace('sdisort', 'disort')
                                newfile3.write(data8) 
                            else:
                                newfile3.write(data7)
                            newfile3.close()                        
                    
                    ### Diffuse radiation
                    elif m == 1:
                        print ("Making the input for sza = %s degrees, diffuse"%(sza[i]))
                        data1 = open('%s/%s'%(libradtran_base,libradtrantemplate), 'r+b')
                        data2 = data1.read()
                        data1.close()
                        data3 = data2.replace('SZA', '%s'%(sza[i]))
                        data4 = data3.replace('ALB', '%s/%s%s.ALB'%(alb_file_location,alb_file_name,sza[np.where(sza == cloudyangle)[0][0]])) 
                        data5= data4.replace('MOLMODIFY', '#')
                        data6 = data5.replace('WCDATA', '%s/WC.DAT'%(locationfiles))
                        data7 = data6.replace('ICDATA', '%s/IC.DAT'%(locationfiles))
                        if verbose == 1:
                            data7 = data7.replace('quiet', 'verbose')
                    
                        newfile3 = open("%s/%s%s.INP"%(locationfiles,basename,sza[i]), 'w')
                        if lwplist[k] >= 0.5 or iwplist[j] >= 0.5:      # Use disort instead of sdisort for thick clouds
                            data8 = data7.replace('sdisort', 'disort')
                            newfile3.write(data8)     
                        elif sza[i] == 0.0:                         # Use disort instead of sdisort for sza = 0.0
                            data8 = data7.replace('sdisort', 'disort')
                            newfile3.write(data8) 
                        else:
                            newfile3.write(data7)
                        newfile3.close()

                    
        
if runninglibradtran == 1:                                      # Creating the libradtran output files
    ### Define a function to run disort.
    def running_disort(locationfiles,angle):
        os.chdir("%s"%(locationfiles))
        y = 0
        while y < 1:
            os.chdir("..")
            if os.getcwd() == '/libRadtran-2.0.2/Own_Files':
                y += 1
        if verbose == 0:
            cmd = "../bin/uvspec" + " <" + "%s/%s%s.INP"%(locationfiles,basename,angle) + "> " + "%s/%s%s.out"%(locationfiles,basename,angle)
        elif verbose == 1:
            cmd = "(../bin/uvspec" + " <" + "%s/%s%s.INP"%(locationfiles,basename,angle) + "> " + "%s/%s%s.out)"%(locationfiles,basename,angle) + ">&" + "%s/verbose.txt"%(locationfiles)
        os.system(cmd)

    ### Loop through the desired DISORT files to run
    for k in range(0,len(lwplist)):
        for j in range(0,len(iwplist)):
            for m in range(0,2):
                #### m = 0 is for direct radiation, m = 1 is for diffuse radiation
                if lwplist[k] != 0.0 or iwplist[j] != 0.0:  
                    if m == 0:          # Skip direct radiation for cloudy conditions, as it is not necessary
                        continue
                if m == 0:
                    for x in range(0,len(PWlist)):
                        locationfiles = "%s/lwp%s_iwp%s/direct/PW=%s"%(libradtran_base,lwplist2[k],iwplist2[j],PWlist[x])
                        for i in range(0,len(sza)):
                            print ("Running libradtran: LWP = %s, IWP = %s, sza = %s, direct PW = %s"%(lwplist[k],iwplist[j],sza[i],PWlist[x]))
                            running_disort(locationfiles,sza[i])
                if m == 1:
                    locationfiles = "%s/lwp%s_iwp%s/diffuse"%(libradtran_base,lwplist2[k],iwplist2[j])
                    if lwplist[k] != 0.0 or iwplist[j] != 0.0:  
                        print ("Running libradtran: LWP = %s, IWP = %s"%(lwplist[k],iwplist[j]))
                        running_disort(locationfiles,sza[np.where(sza == cloudyangle)[0][0]])
                    else:
                        for i in range(0,len(sza)):     
                            # Calculate for all angles only for clear-sky conditions
                            print ("Running libradtran: LWP = %s, IWP = %s, sza = %s"%(lwplist[k],iwplist[j],sza[i]))
                            running_disort(locationfiles,sza[i])                 
                
#%%

# COMPUTING REPRESENTATIVE WAVELENGHTS FOR CLEAR-SKY CONDITIONS, DIFFUSE

print ("Initial calculations")

diffspectralintervals = np.diff(spectralintervalsnm)                        # The difference between the spectral intervals.
mudir = np.cos(sza/180.*np.pi)                                              # Calculating mudir for the given sza.
closestlambdadif = np.zeros((len(sza),len(diffspectralintervals)))          # Closest wavelength that represents the weighted mean as determined using libradtran for diffuse radiation.
albclosestdif = np.zeros_like(closestlambdadif)                             # The albedo when we use the determined closest wavelength in tartes for diffuse radiation. Purely diagnostics

for h in range(0,len(lwplist)):
    for p in range(0,len(iwplist)):        
        locationfiles = "%s/lwp%s_iwp%s/diffuse"%(libradtran_base,lwplist2[h],iwplist2[p])      # File location
        
        if lwplist[h] == 0 and iwplist[p] == 0:
            tartes_output_wavelengthdiflist = []
            tartes_output_wavelengthdirlist = [[] for l in range(len(diffspectralintervals))]
    
        #####
            elemirr = np.zeros((len(sza),len(diffspectralintervals)), dtype = 'int')                       # Counter
            tartes_output_wavelengthirrlist = [[] for l in range(len(diffspectralintervals))]
            for m in range(0,len(sza)):
                print ("Running Tartes for sza = %s degrees"%(sza[m]))
                data = np.loadtxt('%s/%s%s.out'%(locationfiles,basename,sza[m]))        # Loading the data
                wavelength = data[:,0]                                  # Reading the libradtran wavelengths
                diffwavelength = np.diff(wavelength)                    # Difference between the wavelengths
                diffwavelengthhalved = diffwavelength/2                 # Middle of the difference
                wavelength_halved = wavelength[:-1] + diffwavelengthhalved            # Middle of the wavelength
                
            
                irr = data[:,2]/1000                                  # Incoming irradiance according to libradtran
                irrlambda = irr[:-1] * diffwavelength
                meanirr = np.zeros(len(diffspectralintervals))          # Mean radiation
                wmalbirr = np.zeros_like(meanirr)                       # Weighted mean albedo
                irrinterval = []
                
                
                for j in range(0,len(diffspectralintervals)):
                    conditioninterval = (wavelength_halved <= spectralintervalsnm[j+1]) & (wavelength_halved >= spectralintervalsnm[j])
                    wavelengthinterval = wavelength_halved[conditioninterval]
                    irrinterval.append(irrlambda[conditioninterval])
                    weightirr = irrinterval[j]/sum(irrinterval[j])
                
                    ### CALLING TARTES AND COMPUTING ALBEDOS
                    tartes_output_wavelengthirr = tartes.tartes(wavelengthinterval* 10**(-9), ssa,density, thickness, impurities = impurities, impurities_type = impurity_type)
                    tartes_output_wavelengthirrlist[j].append(tartes_output_wavelengthirr)
                    meanirr[j] = np.mean(tartes_output_wavelengthirr)
                    
                    ### COMPUTING THE WEIGHTED MEAN ALBEDO
                    wmalbirr[j] = np.sum(tartes_output_wavelengthirr*weightirr)

                for j in range(0,len(diffspectralintervals)):
                    conditioninterval = (wavelength_halved <= spectralintervalsnm[j+1]) & (wavelength_halved >= spectralintervalsnm[j])
                    wavelengthinterval = wavelength_halved[conditioninterval]
                    if np.isnan(wmalbirr[j]):
                        wmalbirr[j] = meanirr[j]
                    tartes_output_wavelengthirr = tartes_output_wavelengthirrlist[j][m]
                    findingminimum = np.diff((tartes_output_wavelengthirr - wmalbirr[j] > 0) *1)
                
                    if not np.any(findingminimum) == True:
                        findingminimum[0] = -1
                    
                    ### FINDING THE CLOSEST CORRESPONDING WAVELENGTH, OR REPRESENTATIVE WAVELENGTH
                    minimumelement = np.where(findingminimum != 0)[0]
                    minimumelement2 = np.zeros_like(minimumelement)
                    for b in range(0,len(minimumelement)):
                        signal = np.sign(minimumelement[b])
                        menew = []
                        menew.append(np.sign(tartes_output_wavelengthirr - wmalbirr[j])[minimumelement[b]-1])
                        menew.append(np.sign(tartes_output_wavelengthirr - wmalbirr[j])[minimumelement[b]+1])
                        number = np.where(menew != signal)[0][0]                  
                        if number == 0:
                            minimumelement2[b] = minimumelement[b] - 1
                        if number == 1:
                            minimumelement2[b] = minimumelement[b] + 1
                                    
### !!! VERY IMPORTANT !!!, IF STATMENT TO DEFINE WHICH WAVELENGTH TO USE FOR A BAND.
### IT IS IMPORTANT TO KEEP USING THE SAME WAVELENGTH IF MULTIPLE WAVELENGTHS ARE POSSIBLE.
### IT HAS TO BE DEFINED MANUALLY FOR WHICH BANDS ANOTHER WAVELENGTH HAS TO BE USED INSTEAD OF THE FIRST ONE THE ROUTINE FINDS
                                    
                    if len(minimumelement) >= 2:               
                        if j == 8 or j == 9 or j == 10:         ### FOR THESE BANDS, THE SECOND WAVELENGTH WILL BE USED FOR THIS EXAMPLE. 
                            elemirr[m,j] = minimumelement[1]
                            number2 = minimumelement2[1]
                        else:
                            elemirr[m,j] = minimumelement[0]
                            number2 = minimumelement2[0]
                    else:           
                        elemirr[m,j] = minimumelement[0]
                        number2 = minimumelement2[0]
                    
                    #### INTERPOLATING THE CLOSEST WAVELENGTH
                    
                    lambda1 = wavelengthinterval[elemirr[m,j]]
                    lambda2 = wavelengthinterval[number2]
                    albedodiff1 = (tartes_output_wavelengthirr - wmalbirr[j])[elemirr[m,j]]
                    albedodiff2 = (tartes_output_wavelengthirr - wmalbirr[j])[number2]
                    
                    interpolatedlambda = lambda1 - albedodiff1 * ((lambda2 - lambda1)/(albedodiff2 - albedodiff1))
      
                    closestlambdadif[m,j] = interpolatedlambda                    
                    albclosestdif[m,j] = tartes.tartes(closestlambdadif[m,j]* 10**(-9), ssa,density, thickness, impurities = impurities, impurities_type = impurity_type)  
            
            np.savetxt("%s/table_clearsky_diffuse.txt"%(savedir),closestlambdadif)

#%%

# COMPUTING REPRESENTATIVE WAVELENGHTS FOR CLEAR-SKY CONDITIONS, DIRECT

for j in range(0,len(diffspectralintervals)):
    closestlambdadir = np.zeros((len(sza),len(PWlist)))              # Closest wavelength that represents the weighted mean as determined using libradtran for cloudy diffuse radiation.
    albclosestdir = np.zeros_like(closestlambdadir)                     # The albedo when we use the determined closest wavelength in tartes for cloudy diffuse radiation. Purely diagnostics
    sumenergy = np.zeros_like(closestlambdadir)                           # Total energy of the considered wavelength band
    for h in range(0,len(PWlist)):
        for p in range(0,len(sza)):      
            print ("Running Tartes for band = %s, sza = %s, PW = %s"%(j+1,sza[p],PWlist[h]))
            locationfiles = "%s/lwp00_iwp00/direct/PW=%s"%(libradtran_base,PWlist[h])
            elemirr2 = 0
    
            data = np.loadtxt('%s/%s%s.out'%(locationfiles,basename,sza[p]))        # Loading the data
            wavelength = data[:,0]                                          # Reading the libradtran data
            diffwavelength = np.diff(wavelength)                            # Difference between the wavelengths
            diffwavelengthhalved = diffwavelength/2                         # Middle of the difference
            wavelength_halved = wavelength[:-1] + diffwavelengthhalved      # Middle of the wavelength
            
        
            irr = data[:,1]/1000                                            # Incoming irradiance according to libradtran
            irrlambda = irr[:-1] * diffwavelength
            irrinterval = []
            
            
            conditioninterval = (wavelength_halved <= spectralintervalsnm[j+1]) & (wavelength_halved >= spectralintervalsnm[j])
            wavelengthinterval = wavelength_halved[conditioninterval]
            irrinterval.append(irrlambda[conditioninterval])
            weightirr = irrinterval/np.sum(irrinterval)
            sumenergy[p,h] = np.sum(irrinterval)
        
            ### CALLING TARTES AND COMPUTING ALBEDOS
        
            tartes_output_wavelengthirr = tartes_output_wavelengthirr = tartes.tartes(wavelengthinterval* 10**(-9), ssa,density, thickness, mudir = mudir[p], dir_frac = 1.0, impurities = impurities, impurities_type = impurity_type)
            meanirr = np.mean(tartes_output_wavelengthirr)
            
            ### COMPUTING THE WEIGHTED MEAN ALBEDO
            wmalbirr = np.sum(tartes_output_wavelengthirr*weightirr)
    
            if np.isnan(wmalbirr):
                wmalbirr = meanirr
            findingminimum = np.diff((tartes_output_wavelengthirr - wmalbirr > 0) *1)
        
            if not np.any(findingminimum) == True:
                findingminimum[0] = -1
    
            ### FINDING THE CLOSEST CORRESPONDING WAVELENGTH, OR REPRESENTATIVE WAVELENGTH
            minimumelement = np.where(findingminimum != 0)[0]
            minimumelement2 = np.zeros_like(minimumelement)
            for b in range(0,len(minimumelement)):
                signal = np.sign(minimumelement[b])
                menew = []
                menew.append(np.sign(tartes_output_wavelengthirr - wmalbirr)[minimumelement[b]-1])
                menew.append(np.sign(tartes_output_wavelengthirr - wmalbirr)[minimumelement[b]+1])
                number = np.where(menew != signal)[0][0]                  
                if number == 0:
                    minimumelement2[b] = minimumelement[b] - 1
                if number == 1:
                    minimumelement2[b] = minimumelement[b] + 1
                    
### !!! VERY IMPORTANT !!!, IF STATMENT TO DEFINE WHICH WAVELENGTH TO USE FOR A BAND.
### IT IS IMPORTANT TO KEEP USING THE SAME WAVELENGTH IF MULTIPLE WAVELENGTHS ARE POSSIBLE.
### IT HAS TO BE DEFINED MANUALLY FOR WHICH BANDS ANOTHER WAVELENGTH HAS TO BE USED INSTEAD OF THE FIRST ONE THE ROUTINE FINDS                
                
            if len(minimumelement) >= 2:                ### FOR THESE BANDS, THE SECOND WAVELENGTH WILL BE USED FOR THIS EXAMPLE. 
                if j == 8 or j == 9 or j == 10:
                    elemirr2 = minimumelement[1]
                    number2 = minimumelement2[1]
                else:
                    elemirr2 = minimumelement[0]
                    number2 = minimumelement2[0]
            else:           
                elemirr2 = minimumelement[0]
                number2 = minimumelement2[0]
            
            #### INTERPOLATING THE CLOSEST WAVELENGTH
            
            lambda1 = wavelengthinterval[elemirr2]
            lambda2 = wavelengthinterval[number2]
            albedodiff1 = (tartes_output_wavelengthirr - wmalbirr)[elemirr2]
            albedodiff2 = (tartes_output_wavelengthirr - wmalbirr)[number2]
            
            interpolatedlambda = lambda1 - albedodiff1 * ((lambda2 - lambda1)/(albedodiff2 - albedodiff1))
      
            closestlambdadir[p,h] = interpolatedlambda                    
            albclosestdir[p,h] = tartes.tartes(closestlambdadir[p,h]* 10**(-9), ssa,density, thickness, mudir = mudir[p], dir_frac = 1.0, impurities = impurities, impurities_type = impurity_type)    
            


    #### CORRECTING THE ARRAY IF THERE IS NO ENERGY, FILLING IT WITH KNOWN VALUES
    
    sumenergy = np.where(sumenergy < 10**-20, 0, sumenergy)
    truefalse = (sumenergy > 0)*1
    if np.sum(truefalse) == 0:
        truefalse += 1
    
    temp = closestlambdadir * truefalse
    cor_wlarray2 = np.zeros_like(temp)


    for e in range(0,len(PWlist)):
        for i in range(0,len(sza)):
            if temp[i,e] > 0:
                ph = temp[i,e]
                cor_wlarray2[i,e] = ph
            if temp[i,e] == 0 and i != 0:
                cor_wlarray2[i,e] = cor_wlarray2[i-1,e]
            if temp[i,e] == 0 and i == 0:
                break

    for i in range(0,len(sza)):
        for e in range(0,len(PWlist)):
            if cor_wlarray2[i,e] == 0:
                cor_wlarray2[i,e] = cor_wlarray2[i,e-1]
                   
    np.savetxt("%s/band%s_wavelength_direct.txt"%(savedir,j+1),cor_wlarray2)
    

#%%

# COMPUTING REPRESENTATIVE WAVELENGHTS FOR CLOUDY CONDITIONS

for j in range(0,len(diffspectralintervals)):
    closestlambdacloud = np.zeros((len(iwplist),len(lwplist)))              # Closest wavelength that represents the weighted mean as determined using libradtran for cloudy diffuse radiation.
    albclosestcloud = np.zeros_like(closestlambdacloud)                     # The albedo when we use the determined closest wavelength in tartes for cloudy diffuse radiation. Purely diagnostics
    sumenergy = np.zeros_like(closestlambdacloud)                           # Total energy of the considered wavelength band
    for h in range(0,len(lwplist)):
        for p in range(0,len(iwplist)):      
            print ("Running Tartes for band = %s, LWP = %s, IWP = %s"%(j+1,lwplist[h],iwplist[p]))
            locationfiles = "%s/lwp%s_iwp%s/diffuse"%(libradtran_base,lwplist2[h],iwplist2[p])
            elemirr2 = 0
    
            data = np.loadtxt('%s/%s%s.out'%(locationfiles,basename,cloudyangle))        # Loading the data
            wavelength = data[:,0]                                          # Reading the libradtran data
            diffwavelength = np.diff(wavelength)                            # Difference between the wavelengths
            diffwavelengthhalved = diffwavelength/2                         # Middle of the difference
            wavelength_halved = wavelength[:-1] + diffwavelengthhalved      # Middle of the wavelength
            
        
            irr = data[:,2]/1000                                            # Incoming irradiance according to libradtran
            irrlambda = irr[:-1] * diffwavelength
            irrinterval = []
            
            
            conditioninterval = (wavelength_halved <= spectralintervalsnm[j+1]) & (wavelength_halved >= spectralintervalsnm[j])
            wavelengthinterval = wavelength_halved[conditioninterval]
            irrinterval.append(irrlambda[conditioninterval])
            weightirr = irrinterval/np.sum(irrinterval)
            sumenergy[p,h] = np.sum(irrinterval)
        
            ### CALLING TARTES AND COMPUTING ALBEDOS
        
            tartes_output_wavelengthirr = tartes.tartes(wavelengthinterval* 10**(-9), ssa,density, thickness, impurities = impurities, impurities_type = impurity_type)
            meanirr = np.mean(tartes_output_wavelengthirr)
            
            ### COMPUTING THE WEIGHTED MEAN ALBEDO
            wmalbirr = np.sum(tartes_output_wavelengthirr*weightirr)
    
            if np.isnan(wmalbirr):
                wmalbirr = meanirr
            findingminimum = np.diff((tartes_output_wavelengthirr - wmalbirr > 0) *1)
        
            if not np.any(findingminimum) == True:
                findingminimum[0] = -1
    
            ### FINDING THE CLOSEST CORRESPONDING WAVELENGTH, OR REPRESENTATIVE WAVELENGTH
            minimumelement = np.where(findingminimum != 0)[0]
            minimumelement2 = np.zeros_like(minimumelement)
            for b in range(0,len(minimumelement)):
                signal = np.sign(minimumelement[b])
                menew = []
                menew.append(np.sign(tartes_output_wavelengthirr - wmalbirr)[minimumelement[b]-1])
                menew.append(np.sign(tartes_output_wavelengthirr - wmalbirr)[minimumelement[b]+1])
                number = np.where(menew != signal)[0][0]                  
                if number == 0:
                    minimumelement2[b] = minimumelement[b] - 1
                if number == 1:
                    minimumelement2[b] = minimumelement[b] + 1
                    
### !!! VERY IMPORTANT !!!, IF STATMENT TO DEFINE WHICH WAVELENGTH TO USE FOR A BAND.
### IT IS IMPORTANT TO KEEP USING THE SAME WAVELENGTH IF MULTIPLE WAVELENGTHS ARE POSSIBLE.
### IT HAS TO BE DEFINED MANUALLY FOR WHICH BANDS ANOTHER WAVELENGTH HAS TO BE USED INSTEAD OF THE FIRST ONE THE ROUTINE FINDS                
                
            if len(minimumelement) >= 2:                ### FOR THESE BANDS, THE SECOND WAVELENGTH WILL BE USED FOR THIS EXAMPLE. 
                if j == 8 or j == 9 or j == 10:
                    elemirr2 = minimumelement[1]
                    number2 = minimumelement2[1]
                else:
                    elemirr2 = minimumelement[0]
                    number2 = minimumelement2[0]
            else:           
                elemirr2 = minimumelement[0]
                number2 = minimumelement2[0]
            
            #### INTERPOLATING THE CLOSEST WAVELENGTH
            
            lambda1 = wavelengthinterval[elemirr2]
            lambda2 = wavelengthinterval[number2]
            albedodiff1 = (tartes_output_wavelengthirr - wmalbirr)[elemirr2]
            albedodiff2 = (tartes_output_wavelengthirr - wmalbirr)[number2]
            
            interpolatedlambda = lambda1 - albedodiff1 * ((lambda2 - lambda1)/(albedodiff2 - albedodiff1))
      
            closestlambdacloud[p,h] = interpolatedlambda                    
            albclosestcloud[p,h] = tartes.tartes(closestlambdacloud[p,h]* 10**(-9), ssa,density, thickness, impurities = impurities, impurities_type = impurity_type)  
            


    #### CORRECTING THE ARRAY IF THERE IS NO ENERGY, FILLING IT WITH KNOWN VALUES
    
    sumenergy = np.where(sumenergy < 10**-20, 0, sumenergy)
    truefalse = (sumenergy > 0)*1
    if np.sum(truefalse) == 0:
        truefalse += 1
    
    temp = closestlambdacloud * truefalse
    cor_wlarray2 = np.zeros_like(temp)



    for e in range(0,len(lwplist)):
        for i in range(0,len(iwplist)):
            if temp[i,e] > 0:
                ph = temp[i,e]
                cor_wlarray2[i,e] = ph
            if temp[i,e] == 0 and i != 0:
                cor_wlarray2[i,e] = cor_wlarray2[i-1,e]
            if temp[i,e] == 0 and i == 0:
                break

    for i in range(0,len(iwplist)):
        for e in range(0,len(lwplist)):
            if cor_wlarray2[i,e] == 0:
                cor_wlarray2[i,e] = cor_wlarray2[i,e-1]
                   
    np.savetxt("%s/band%s_wavelength_cloudy.txt"%(savedir,j+1),cor_wlarray2)

#%%    

#### COMPUTING THE DIRECT/TOTAL ENERGY FRACTION

dirdiffrac = np.zeros((len(sza),len(diffspectralintervals)))
dirsum = np.zeros_like(dirdiffrac)
difsum = np.zeros_like(dirdiffrac)

for i in range(0,len(sza)):
    locationfiles = "%s/lwp00_iwp00/diffuse"%(libradtran_base)                               # Loading in the diffuse data file, which also has the desired direct data
    data = np.loadtxt('%s/%s%s.out'%(locationfiles,basename,sza[i]))                         # Loading the data
    wavelength = data[:,0]                                                                   # Reading the libradtran data
    for j in range(0,len(diffspectralintervals)):
        conditioninterval = (wavelength <= spectralintervalsnm[j+1]) & (wavelength >= spectralintervalsnm[j])
        datainterval = data[conditioninterval]
        dirdiffrac[i,j] = np.nansum(datainterval[:,1])/np.sum(datainterval[:,2])
        dirsum[i,j] = np.nansum(datainterval[:,1])
        difsum[i,j] = np.sum(datainterval[:,2])

dirdifsum = dirsum + difsum
dirfrac = dirsum/dirdifsum
dirfrac[:,0] = dirfrac[:,1] 

dirdiffrac[:,0] = dirdiffrac[:,1]  

np.savetxt("%s/dirsum_table.txt"%(savedir),dirfrac)
