"""
Script to calculate the albedo using TARTES with lookup tables from SNOWBALv1.2 and conditions according to RACMO 
for a specific point on the Greenland ice sheet during a year. Data is considered at the same moment every day, e.g. 15:00 GMT.
Output: A time series figure with multiple parameters.

MADE BY C.T. van Dalum et al. (2018, 2019)
"""


#### Packages ####

import tartes
import matplotlib.pyplot as plt
import os.path
import seaborn as sns
import numpy as np
import netCDF4 as S
from scipy import interpolate
import time
from matplotlib import gridspec
import matplotlib.patches as patches

import sys
sys.path.insert(0, '/Scripts/')                                                     # Set the system to the right path, might not be required

degree_sign= u'\N{DEGREE SIGN}'   

sns.set()
sns.set(font_scale=1.1)
sns.set_style("ticks", {"xtick.major.size": 5, "ytick.major.size": 5})

###
# INPUT
###   

datalocatie = "/RACMO_runs/netcdf/"                                                 # Directory with data (in netcdf format), all data in this directory will be loaded
lookuptablelocatie = "/Lookup_tables_RACMO/"                                        # Directory with the lookup tables for RACMO
savedir = "/Results/snowbal1_2/"                                                    # Save directory

lwpthres = 0.05                                                                     # LWP threshold. It is assumed that all radiation is cloudy diffuse beyond this value
iwpthres = 0.01                                                                     # IWP threshold. It is assumed that all radiation is cloudy diffuse beyond this value
impcon = 0.0 *10**(-9)                                                              # Impurities concentration in kg/kg (one value if constant, otherwise a value for all layers) #DEFAULT SOOT
ssabareice = 0.08                                                                   # SSA of the bare ice, taken as a constant for all conditions
wlbands = 12                                                                        # Number of wavelength bands

anglescs = np.linspace(0,90,90/2.5 + 1)                                             # Angles to use for libradtran and tartes for the solar zenith angle of direct radiation.
anglescs[-1] = 89.9
angleswl = np.linspace(0,53./9.*6. + 53.,16)                                        # SZA of the lookuptable for wavelength determination of clear sky conditions for direct and diffuse.
iwplist = np.array([0.0,0.01, 0.05, 0.1, 0.15, 0.2, 0.5, 1.0, 2.0,5.0])             # Ice water path for the lookup table interpolation
lwplist = np.array([0.0,0.05,0.1,0.15,0.2,0.5,1.0,3.0,5.0,10.0,20.0,40.0])          # Liquid water path for the lookup table interpolation
PW_list = np.array([0.5,2.0,4.0,6.0,8.0,10.0,13.0,18.0,25.0,40.0])                  # Precipitable water for the lookup table interpolation

lw = 1.5                                                                            # Linewidth for plots
year = 2007                                                                         # Considered year
timegmt = 1500                                                                      # Time in GMT
    
latpoint = 46                                                                       # Latitude element of the conidered point   (for latpoint = 46: 64.67 N)
lonpoint = 40                                                                       # Longitude element of the considered point (for lonpoint = 40: 45.05 W)

pkm = 1                                                                             # 1 If the albedo according to Kuipers Munneke et al. (2011) is considered
plot = 1                                                                            # 1 If plots are made     

rhoice = 917.0                                                                      # Density of ice
albbareice = 0.42                                                                   # Bare ice albedo (For PKM)


# Plot variables

monthlist = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"]      # Months
monthlistabb = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", "Jan"]                                  # Months abbreviation
monthselementen = np.array([31,28,31,30,31,30,31,31,30,31,30,31])                                                                           # Days in the month
  
  
### LOADING DATA
    
lambdaclouds = []
lambdadir = []

for i in range(0,wlbands):
    lambdaclouds.append(np.loadtxt("%s/band%s_wavelength_cloudy.txt"%(lookuptablelocatie,i+1)))         # Wavelength for cloudy conditions for the 12 bands. 
    lambdadir.append(np.loadtxt("%s/band%s_wavelength_direct.txt"%(lookuptablelocatie,i+1)))            # Wavelength Clear sky direct radiation lookup table

lambdadif = np.loadtxt("%s/table_clearsky_diffuse.txt"%(lookuptablelocatie))                            # Wavelength Clear sky diffuse radiation lookup table
dirsumfrac = np.loadtxt("%s/dirsum_table.txt"%(lookuptablelocatie))                                     # Direct/total energy fraction per band and as a function of sza.
     
  
#%%
    
############
#### Loading data sets and producing the interpolation functions.
############
    
#Making a list of data files:
datalist = sorted(os.listdir(datalocatie))

if datalist[0] == '.DS_Store':
    del datalist[0]

datalistplot = []
for i in range(0,len(datalist)):
    datalistplot.append(datalist[i].replace(".nc",""))

lambdaclouds = np.array(lambdaclouds)
lambdadir = np.array(lambdadir)                                                           

if not os.path.exists(savedir):                                                     # Making the save directory if it does not exist
    os.makedirs(savedir)   
  
#######

angleswlnew = []
lambdadirnew = []
lambdadifnew = []
for i in range(0,len(angleswl)):
    angleswlnew.append(angleswl[i])
angleswlnew.append(90.0)
angleswlnew = np.array(angleswlnew)

for j in range(0,wlbands):
    for i in range(0,len(angleswl)):
        lambdadifnew.append(lambdadif[i,j])
    lambdadifnew.append(lambdadif[i,j])
lambdadifnew = np.array(lambdadifnew).reshape(wlbands,len(angleswl)+1)
####

####
lambdadirnew = []
for j in range(0,wlbands):
    for k in range(0,len(PW_list)):
        for i in range(0,len(angleswl)):
            lambdadirnew.append(lambdadir[j,i,k])
        lambdadirnew.append(lambdadir[j,i,k])
lambdadirnew = np.array(lambdadirnew).reshape(wlbands,len(PW_list),len(angleswl)+1)

temp = []
for j in range(0,wlbands):
    temp.append(np.transpose(lambdadirnew[j]))
lambdadirnew = np.array(temp)
###

cloudsinterpolate = []
directfraction = []
wavelengthdirinterpolate = []
wavelengthdifinterpolate = []
for i in range(0,wlbands):
    cloudsinterpolate.append(interpolate.interp2d(lwplist,iwplist,lambdaclouds[i]))                 # Interpolated wavelength for a certain lwp (x axis) and iwp (y axis)
    directfraction.append(interpolate.interp1d(anglescs,dirsumfrac[:,i]))                           # Interpolated direct/sum fraction
    wavelengthdirinterpolate.append(interpolate.interp2d(PW_list,angleswlnew,lambdadirnew[i]))      # Interpolated direct radiation wavelength for a certain sza and PW, when no clouds are present
    wavelengthdifinterpolate.append(interpolate.interp1d(angleswlnew,lambdadifnew[i]))              # Interpolated diffuse radiation wavelength for a certain sza, when no clouds are present
#######

##  Reading data, calculations and making plots

albBB_TART_time = []
albBB_RACMO_time = []
albBB_PKM_time = []
albBB_diff_TART_RACMO = []
albBB_diff_TART_PKM = []
cflist = []
t2mlist = []
spectralSWdownlist = []
SWdownlist = []
szalist = []
ssa_time = []
thickness_time = []
meltflux_time = []

for i in range(0,len(datalistplot)):
    print ("Starting with %s"%(datalistplot[i]))
    data = S.Dataset("%s/%s"%(datalocatie,datalist[i]))

    totaldown = data.variables["var176"][0,2:16][:,latpoint,lonpoint]       ### Here is the band dependent flux   
    totalup = data.variables["var176"][0,16:30][:,latpoint,lonpoint]        ### Here is the band dependent flux
    cleardown = data.variables["var176"][0,30:44][:,latpoint,lonpoint]      ### Here is the band dependent flux
    clearup = data.variables["var176"][0,44:58][:,latpoint,lonpoint]        ### Here is the band dependent flux
    
    SWdown = data.variables["var176"][0,1][latpoint,lonpoint]               ### total SW down
    
    spectralSWdownlist.append(totaldown)
    SWdownlist.append(SWdown)

#####    
    frac_totaldown = totaldown/np.sum(totaldown,axis=0)                     # Fraction of the total energy
    frac_totaldown = np.where(np.isnan(frac_totaldown),0,frac_totaldown)    # Set nan to 0. 
    
    #### Atmospheric variables and others
    cf = data.variables["var71"][0,0][latpoint,lonpoint]     # Cloud fraction
    lwp = data.variables["var76"][0,0][latpoint,lonpoint]    # Liquid water Path, is a gridbox average!!! (Not necessarely the same as the cloud)
    iwp = data.variables["var77"][0,0][latpoint,lonpoint]    # Ice water Path, is a gridbox average!!! (Not necessarely the same as the cloud)
    t2m = data.variables["var11"][0,1][latpoint,lonpoint]    # 2 meter temperature
    sza = np.degrees(np.arccos(data.variables["var172"][0,0][latpoint,lonpoint]))    # SZA
    mudir = np.cos(sza/180.*np.pi)                # mu
    mudir = np.where(mudir < 0, 0, mudir)
    icemask = np.where(data.variables["var91_2"][0,0][latpoint,lonpoint] > 0.5, 1, data.variables["var91_2"][0,0][latpoint,lonpoint])   #ice mask
    icemask = np.where(icemask < 0.5, 0.0, icemask)
    bgalbedo = data.variables["var84"][0,0][latpoint,lonpoint]*icemask     # Background albedo of ice/snow grid points
    pw_racmo = data.variables["var230"][0,0][latpoint,lonpoint]*icemask    # Precipitable water, which is var230 in RAMCO2.3p2
    
    albedoracmo = np.sum(totalup,axis=0)/np.sum(totaldown,axis=0)          # Albedo in RACMO
    albedoracmo = np.where(albedoracmo == -np.inf, 0, albedoracmo)
    albedoracmo = np.where(albedoracmo == np.inf, 0, albedoracmo)
    albedoracmo = np.where(np.isnan(albedoracmo), 0, albedoracmo)
    albedosnowupdownracmo = albedoracmo*icemask
    albedosnowracmo = data.variables["var156"][0,-1][latpoint,lonpoint]*icemask

    meltflux = data.variables["var218"][0,0][latpoint,lonpoint]       # Meltflux
    
    meltflux_time.append(meltflux)
    cflist.append(cf)
    t2mlist.append(t2m)
    szalist.append(sza)

    #### CORRECTING THE LWP AND IWP FOR FRACTIONAL CLOUD COVER CASES.
    cfinverted = 1/cf
    cfinverted = np.where(cfinverted==np.inf,0,cfinverted)     
    lwpcorrected = lwp*cfinverted           ## Correcting the lwp using the cloud fraction. LWP does not correspond to the actual lwp of the cloud, Because lwp is a gridbox average
    iwpcorrected = iwp*cfinverted           ## Correcting the iwp using the cloud fraction. IWP does not correspond to the actual lwp of the cloud, Because iwp is a gridbox average

    ### Setting the lwp and iwp to 0 if the cloud cover is very low, to prevent excessive lwp and iwp values.
    lwpcorrected = np.where(cf<0.05,0,lwpcorrected)
    iwpcorrected = np.where(cf<0.05,0,iwpcorrected)
    
    ### COMPUTING THE REPRESENTATIVE WAVELENGTHS

    wlinter = np.zeros(wlbands)
    wavelengthdir = np.zeros(wlbands)
    wavelengthdif = np.zeros(wlbands)
    dirfrac = []
    for k in range(0,wlbands):
        dirfrac.append(directfraction[k](sza))              
        wlinter[k] = cloudsinterpolate[k](lwpcorrected,iwpcorrected)            # Interpolated cloud wavelength
        wavelengthdir[k] = wavelengthdirinterpolate[k](pw_racmo,sza)            # Interpolated direct wavelength
        wavelengthdif[k] = wavelengthdifinterpolate[k](sza)                     # Interpolated diffuse wavelength
    dirfrac = np.array(dirfrac)

    #### CORRECTING THE ENERGY FLUX FOR VERY THIN CLOUDS

    lwpfunction = interpolate.interp1d([0,lwpthres], [0,1])
    iwpfunction = interpolate.interp1d([0,iwpthres], [0,1])
    weightclouds_lwp = np.where(lwpcorrected > lwpthres, lwpthres, lwpcorrected)
    weightclouds_iwp = np.where(iwpcorrected > iwpthres, iwpthres, iwpcorrected)
    
    weightclouds_lwp2 = lwpfunction(weightclouds_lwp)
    weightclouds_iwp2 = iwpfunction(weightclouds_iwp)
    weightclouds_lwp3 = weightclouds_lwp2 * cf          ## Sort of 'weighted' cloud cover by lwp. It shows the fraction of "Clear-sky" radation for very thin clouds. This is because very thin clouds let (direct) radiation penetrate, 
                                                        ## so we can see a part of the radiation with these conditions as a "clear sky" radiation, on top of "clear sky" radiation fraction due to the total cloud cover.
    weightclouds_iwp3 = weightclouds_iwp2 * cf

    newcf = np.where(weightclouds_lwp3 >= weightclouds_iwp3, weightclouds_lwp3, weightclouds_lwp3)      # New "weighted" cloud cover 
    newcf = np.where(newcf < weightclouds_iwp3, weightclouds_iwp3, newcf) 

    fluxdifd_clouds = newcf * totaldown[:-2]                        # Diffuse downward flux due to clouds (penetration through thin clouds is included in clear sky part)
    fluxdifu_clouds = newcf * totalup[:-2]                          # Diffuse upward flux due to clouds
    fluxdird_clear = (1 - newcf) * dirfrac * totaldown[:-2]         # Direct downward flux clear sky
    fluxdiru_clear = (1 - newcf) * dirfrac * totalup[:-2]           # Direct upward flux clear sky
    fluxdifd_clear = (1 - newcf) * (1-dirfrac) * totaldown[:-2]     # Diffuse downward flux clear sky
    fluxdifu_clear = (1 - newcf) * (1-dirfrac) * totalup[:-2]       # Diffuse upward flux clear sky

    #### FRACTION OF ENERGY WITH RESPECT TO THE TOTAL FOR THE FIRST 12 BANDS

    frac_fluxdifd_clouds = fluxdifd_clouds/totaldown[:-2]
    frac_fluxdifu_clouds = fluxdifu_clouds/totaldown[:-2]  
    frac_fluxdird_clear = fluxdird_clear/totaldown[:-2]  
    frac_fluxdiru_clear = fluxdiru_clear/totaldown[:-2]  
    frac_fluxdifd_clear = fluxdifd_clear/totaldown[:-2]  
    frac_fluxdifu_clear = fluxdifu_clear/totaldown[:-2]  
    
    frac_fluxdifd_clouds = np.where(np.isnan(frac_fluxdifd_clouds), 0, frac_fluxdifd_clouds)
    frac_fluxdifu_clouds = np.where(np.isnan(frac_fluxdifu_clouds), 0, frac_fluxdifu_clouds)  
    frac_fluxdird_clear = np.where(np.isnan(frac_fluxdird_clear), 0, frac_fluxdird_clear)  
    frac_fluxdiru_clear = np.where(np.isnan(frac_fluxdiru_clear), 0, frac_fluxdiru_clear)  
    frac_fluxdifd_clear = np.where(np.isnan(frac_fluxdifd_clear), 0, frac_fluxdifd_clear)  
    frac_fluxdifu_clear = np.where(np.isnan(frac_fluxdifu_clear), 0, frac_fluxdifu_clear)  
    
    #####
    # SNOW PROFILE CALCULATIONS
    #####

    ### Snow profile variables (With the top 20 layers in depth (A fraction of the top most layer is fresh snow, which is calculated seperately))
    thickness = data.variables["var67"][0][:,latpoint,lonpoint]                # Layer thickness
    density = data.variables["var157"][0][:,latpoint,lonpoint]                 # Density of the snow layers
    radius = data.variables["var154"][0][:,latpoint,lonpoint]                  # Particle radius
    ssa = (3/rhoice)/radius                                                    # Converting to SSA
    ssa2 = np.where(ssa==np.inf,0,ssa)                                         # Putting non snow/ice tiles to 0.
    radiusfresh = data.variables["var153"][0,0][latpoint,lonpoint]             # Fresh snow layer grain size
    massfresh = data.variables["var70"][0,0][latpoint,lonpoint]                # Fresh snow layer mass
    densityfresh = data.variables["var156"][0,-2][latpoint,lonpoint]           # Fresh snow layer density
    ssafresh = (3/rhoice)/radiusfresh                                          # SSA fresh snow
    ssafresh2 = np.where(ssafresh==np.inf,0,ssafresh)                          # Putting non snow/ice tiles to 0.
    
    ### Calculating the thickness of the fresh snow layer.
    thicknessfresh = massfresh / densityfresh
    thicknessfresh2 = np.where(thicknessfresh==-np.inf,0,thicknessfresh)
    thicknessfresh2 = np.where(thicknessfresh2==np.inf,0,thicknessfresh2)
    thicknessfresh2 = np.where(np.isnan(thicknessfresh2),0,thicknessfresh2)
    thicknessfresh2 = np.where(thicknessfresh2 < 0, 0, thicknessfresh2)
    
    ### ADDING THE FRESH SNOW LAYER AND ADJUSTING THE THICKNESS OF THE LAYER UNDER THE FRESH SNOW (PREVIOUSLY THE FIRST LAYER).
    thickness[0] = thickness[0] - thicknessfresh2
    newthickness = []
    newssa = []
    newdensity = []
    newradius = []
    
    #### New lists with 20 + 1 fresh snow layer on top. 
    newthickness.append([thicknessfresh2][0])
    newssa.append([ssafresh2][0])
    newdensity.append([densityfresh][0])
    newradius.append([radiusfresh][0])
    
    for k in range(0,20):                               # 20, because the output of RACMO2.3p2 is limited to the upper 20 layers
        newthickness.append(list(thickness)[k])
        newssa.append(list(ssa2)[k])
        newdensity.append(list(density)[k])
        newradius.append(list(radius)[k])
    newthickness = np.array(newthickness)
    newssa = np.array(newssa)
    newdensity = np.array(newdensity)
    newradius = np.array(newradius)
        
    ## ADJUSTING THE SSA FOR BARE ICE, SO THAT IT HAS A BROADBAND ALBEDO OF APPROXIMATELY 0.4.          NEW BARE ICE SCHEME
    newssa2 = []
    for a in range(1,len(newssa)+1):    
        if -a == -1:
            newssa2.append(np.where(newdensity[-a] > 888,ssabareice,newssa[-a]))
        else:
            temp1 = newdensity[-a] > 888
            temp2 = newssa2[a-2] == ssabareice
            temp3 = temp1*1.0 + temp2*1.0
            newssa2.append(np.where(temp3 == 2, ssabareice, newssa[-a]))
    newssa = np.array(newssa2)[::-1]
    
    
    ### CALCULATING THE ALBEDO USING TARTES
    albedodir = np.zeros_like(wlinter)          # Clear sky Direct albedo using TARTES
    albedodif = np.zeros_like(wlinter)          # Clear sky Diffuse albedo using TARTES
    albedoclouded = np.zeros_like(wlinter)      # Clouded Diffuse albedo using TARTES
    start_time = time.time()
    if np.sum(newthickness[:]) != 0 and mudir != 0:
        albedodir[:] = tartes.tartes(wavelengthdir[:]* 10**(-9), newssa[:],newdensity[:], newthickness[:],impurities = impcon, soilalbedo=bgalbedo, mudir = mudir, dir_frac = 1.0)
        albedodif[:] = tartes.tartes(wavelengthdif[:]* 10**(-9), newssa[:],newdensity[:], newthickness[:],impurities = impcon,soilalbedo=bgalbedo)
        albedoclouded[:] = tartes.tartes(wlinter[:]* 10**(-9), newssa[:],newdensity[:], newthickness[:],impurities = impcon,soilalbedo=bgalbedo)    
    albedodir = np.where(albedodir < 0.0, 0, albedodir)                 ## Correcting for any negative albedo
    albedodif = np.where(albedodif < 0.0, 0, albedodif)  
    albedoclouded = np.where(albedoclouded < 0.0, 0, albedoclouded) 
    
    albedoweightdir = albedodir*frac_fluxdird_clear
    albedoweightdif = albedodif*frac_fluxdifd_clear
    albedoweightclouded = albedoclouded*frac_fluxdifd_clouds
    
    albedo_TARTES = albedoweightdir + albedoweightdif + albedoweightclouded             # Resulting albedo after weighting CS direct and diffuse, and clouded diffuse.
    albedo_TARTES = list(albedo_TARTES)
    albedo_TARTES.append(0)
    albedo_TARTES.append(0)
    
    ####  END RESULT, SO THE ALBEDO AND BROADBAND ALBEDO ACCORDING TO TARTES
    
    albedo_TARTES = np.array(albedo_TARTES)
    albedo_TARTES_broadband = np.sum(albedo_TARTES*frac_totaldown,axis=0)
    


    if albedo_TARTES_broadband == 0:
        albedodifference = 0.0
        albedosnowupdownracmo = 0.0 
    else:
        albedodifference = albedo_TARTES_broadband-albedosnowupdownracmo
    if icemask == 0:
        albedodifference = np.nan
    if sza == 90:
        albedodifference = np.nan

    albBB_TART_time.append(albedo_TARTES_broadband)
    albBB_RACMO_time.append(albedosnowupdownracmo)
    albBB_diff_TART_RACMO.append(albedodifference)
    ssa_time.append(newssa)
    thickness_time.append(newthickness)
    


###
### PKM
####


    if pkm == 1:
        alb_pkm = np.zeros_like(cf)
        xmu0 = np.zeros_like(mudir)
        da_c = np.zeros_like(mudir)
        mass = newthickness * newdensity            #Ice mass
        zmwe = 0                # Starting depth  
        alb_layer = np.zeros_like(newradius)
        
        newradius = 3/rhoice/newssa             # Using the same bare ice SSA (but now converted to a radius) as before. 
        
        surfpres = data.variables["var1"][0,0][latpoint,lonpoint]           # Surface pressure [Pa] , only necessary for the computation of the albedo of PKM
        cldoptthick = data.variables["var156"][0,36][latpoint,lonpoint]     # Cloud optical thickness, only necessary for the computation of the albedo of PKM.

        ## Correcting cloud optical thickness for fractional cloud cover
        cldoptthick = cldoptthick * cfinverted        
        
        test = np.sqrt(cldoptthick/(3.0*mudir))
        test = np.where(test > 1.0, 1.0, test)
        xmu0 = np.where(mudir > 0.1*10**-8, test, xmu0)
        
        
        da_h = 0.03247*np.log(surfpres/153880.0)
        da_h = np.where(da_h > 0.0, 0.0, da_h)
        da_h = np.where((surfpres < 40000.0), 0.0, da_h)       
        da_h = np.where((surfpres > 110000.0), 0.0, da_h)  
            
    
        uprime = 0.64*xmu0 + (1.0-xmu0)*mudir
        GSEQ9F = 0.53*((1.0-uprime)**1.2)
                
        
        for l in range(0,len(newthickness[:])):
        
            a_s = 1.48 - 1.27048*((newradius[l])**0.07)
            da_c = 0.04 - a_s 
            da_c2 = 0.0             # Valid only when there are no impurities
            da_c = np.where(da_c < da_c2, da_c2, da_c)  
            a_c = a_s + da_c
            
            da_upr = a_s*(1.0-a_c)*GSEQ9F
            
            da_tau= (0.1*cldoptthick*(a_c**1.3)) / ((1.0+1.5*cldoptthick)**a_s) 
            
            som = a_c+da_upr+da_tau+da_h
            som = np.where(som > 0.98, 0.98, som)
            alb_layer[l] = np.where(som < albbareice, albbareice, som)        # Background albedo is bare ice.
         
      
        alb_layer2 = []
        for a in range(1,len(alb_layer)+1):    
            if -a == -1:
                alb_layer2.append(np.where(newdensity[-a] > 888,albbareice,alb_layer[-a]))
            else:
                testje1 = newdensity[-a] > 888
                testje2 = alb_layer2[a-2] == albbareice
                testje3 = testje1*1.0 + testje2*1.0
                alb_layer2.append(np.where(testje3 == 2, albbareice, alb_layer[-a]))
        alb_layer = np.array(alb_layer2)[::-1]
    
        zmwetruefalse = []                                              # Array used to determine when more than 10 cmwe of snow/ice is considered.
        for l in range(0,len(newthickness[:])):
            zmwetruefalse.append(zmwe < 0.1)                            # Only consider the top 10 cmwe of snow/ice
            if l == 0:
                alb_pkm += alb_layer[l]
            else:
                alb_pkm += zmwetruefalse[l] * (alb_layer[l] - alb_layer[l-1])* np.exp(-zmwe/0.01)
            
            zmwe = zmwe + mass[l]/1000.0            # 1000 kg/m3 is the density of water. 

        if albedo_TARTES_broadband == 0:
            albedodifference_PKM = 0.0
            alb_pkm = 0.0
        else:
            albedodifference_PKM = albedo_TARTES_broadband-alb_pkm
        if icemask == 0:
            albedodifference_PKM = np.nan
        if sza == 90:
            albedodifference_PKM = np.nan            
        
        albBB_PKM_time.append(alb_pkm)
        albBB_diff_TART_PKM.append(albedodifference_PKM)

#####
## Converting lists to arrays
#####

albBB_TART_time = np.array(albBB_TART_time)
albBB_RACMO_time = np.array(albBB_RACMO_time)
albBB_PKM_time = np.array(albBB_PKM_time)
albBB_diff_TART_RACMO = np.array(albBB_diff_TART_RACMO)
albBB_diff_TART_PKM = np.array(albBB_diff_TART_PKM)

albBB_TART_time = np.ma.masked_where(albBB_TART_time == 0,albBB_TART_time)
albBB_RACMO_time = np.ma.masked_where(albBB_RACMO_time == 0,albBB_RACMO_time)
albBB_PKM_time = np.ma.masked_where(albBB_PKM_time == 0,albBB_PKM_time)
albBB_diff_TART_RACMO = np.ma.masked_where(np.isnan(albBB_diff_TART_RACMO),albBB_diff_TART_RACMO)
albBB_diff_TART_PKM = np.ma.masked_where(np.isnan(albBB_diff_TART_PKM),albBB_diff_TART_PKM)

cflist = np.array(cflist)
t2mlist = np.array(t2mlist)
szalist = np.array(szalist)
SWdownlist = np.array(SWdownlist)
spectralSWdownlist = np.array(spectralSWdownlist)
thickness_time = np.array(thickness_time)
ssa_time = np.array(ssa_time)
meltflux_time = np.array(meltflux_time)

vis_list = np.sum(spectralSWdownlist[:,0:5],axis = 1)
ir_list = np.sum(spectralSWdownlist[:,5:-1],axis = 1)
    

ssa_time2 = ssa_time.transpose()
thickness_time2 = thickness_time.transpose()

#%%

######
    # PLOTTING
######   

if plot == 1:

    monthscumsum = np.cumsum(monthselementen)
    monthscumsum2 = monthscumsum + 5
    
    test = []
    for i in range(0,len(monthscumsum2)-1):
        test.append(datalistplot[monthscumsum2[i]])
        
    
    xlist = np.zeros(13)   
    for i in range(0,len(xlist)-1):
        xlist[i+1] = monthscumsum[i]
        
    nulline = np.zeros_like(albBB_TART_time)
    xdays = np.linspace(0,len(albBB_TART_time)-1,len(albBB_TART_time))

   
    ## Starting the figure
    fig = plt.figure(figsize = (7,9))
    
    if pkm == 1:
        gs = gridspec.GridSpec(8, 1, height_ratios=[2.4, 2.4, 1, 1, 1, 1, 1,2.4]) 
    else:
        gs = gridspec.GridSpec(5, 1, height_ratios=[2, 2, 1, 1, 1]) 
    
    axs = plt.subplot(gs[0])
    axs2 = plt.subplot(gs[1],sharex = axs)
    axs3 = axs2.twinx()
    if pkm == 1:
        axs4 = plt.subplot(gs[2],sharex = axs)
        axs5 = plt.subplot(gs[3],sharex = axs)
        axs6 = axs5.twinx()
        axs7 = plt.subplot(gs[4],sharex = axs)
        axs8 = plt.subplot(gs[5],sharex = axs)
        axs9 = axs8.twinx()
        axs10 = plt.subplot(gs[6],sharex = axs)
        axs11 = plt.subplot(gs[7],sharex = axs)
        axs12 = axs11.twinx()
    else:
        axs4 = plt.subplot(gs[2],sharex = axs)
        axs5 = plt.subplot(gs[3],sharex = axs)
        axs6 = axs5.twinx()
        axs7 = plt.subplot(gs[4],sharex = axs)
        axs8 = plt.subplot(gs[5],sharex = axs)
        axs9 = axs8.twinx()       

    #### Subplot 1 SW
    
    axs.plot(SWdownlist, label = "Total",linewidth = lw,color='black',alpha = 0.8)
    axs.plot(vis_list, label = "UV + Visible",linewidth = lw,color='blue',alpha = 0.8)
    axs.plot(ir_list, label = "IR",linewidth = lw,color='red',alpha = 0.8)
    
    axs.set_ylim(0,1000)
    axs.text(-9, 420, '(a)', fontsize=12, weight = 'bold',backgroundcolor='w')
    axs.legend(loc = 'upper left')
    axs.set_ylabel("Surface Downward \n SW Radiation [W m$^{-2}$]")    
    
    ##### Subplot 2   BROADBAND ALBEDO
    
    axs3.plot(albBB_TART_time, label = "TARTES",linewidth = lw,color='red',alpha = 0.7)
    axs3.scatter(365 + 18,np.mean(albBB_TART_time),marker="_",color = 'red', linewidth = 2.5, s = 550,alpha = 0.7)
    axs3.plot(albBB_RACMO_time, label = "RACMO2",linewidth = lw,color='blue',alpha = 0.7)
    axs3.scatter(365 + 18,np.mean(albBB_RACMO_time),marker="_",color = 'blue', linewidth = 2.5, s = 550,alpha = 0.7)
    if pkm == 1:
        axs3.plot(albBB_PKM_time, label = "PKM", linewidth = lw, color = 'green',alpha = 0.7)
        axs3.scatter(365 + 18,np.mean(albBB_PKM_time),marker="_",color = 'green', linewidth = 2.5, s = 550,alpha = 0.7)
    
    axs3.set_ylim(0.6,1.0)
    axs3.text(-9, 0.95, '(b)', fontsize=12, weight = 'bold',backgroundcolor='w')
    axs3.set_ylabel("Broadband Albedo")
    axs3.legend(loc = "lower right")
    axs2.axes.get_yaxis().set_visible(False)
    
    #### Subplot 3  TARTES - RACMO
    
    axs4.plot(albBB_diff_TART_RACMO, label = "TARTES - RACMO2",linewidth = lw,color='black',alpha = 0.8)
    axs4.plot(nulline, linewidth = lw, color = "black", alpha = 0.8, ls = "--")
    axs4.fill_between(xdays, nulline, albBB_diff_TART_RACMO, where=(nulline > albBB_diff_TART_RACMO), facecolor='r', edgecolor='k', alpha=0.5)
    axs4.fill_between(xdays, nulline, albBB_diff_TART_RACMO, where=(nulline < albBB_diff_TART_RACMO), facecolor='g', edgecolor='k', alpha=0.5)
    
    axs4.set_ylim(-0.1,0.1)
    axs4.text(-9, 0.04, '(c)', fontsize=12, weight = 'bold',backgroundcolor='w')
    axs4.set_ylabel("Albedo Difference: \n TARTES - RACMO2")
    
    
    #### Subplot 4  TARTES - PKM
    if pkm == 1:
        
        axs6.plot(albBB_diff_TART_PKM, label = "TARTES - PKM", linewidth = lw, color = 'black',alpha = 0.8)
        axs6.plot(nulline, linewidth = lw, color = "black", alpha = 0.8, ls = "--")
        axs6.fill_between(xdays, nulline, albBB_diff_TART_PKM, where=(nulline > albBB_diff_TART_PKM), facecolor='r', edgecolor='k', alpha=0.5)
        axs6.fill_between(xdays, nulline, albBB_diff_TART_PKM, where=(nulline < albBB_diff_TART_PKM), facecolor='g', edgecolor='k', alpha=0.5)
        
        axs6.set_ylim(-0.1,0.1)
        axs6.text(-9, 0.04, '(d)', fontsize=12, weight = 'bold',backgroundcolor='w')
        axs6.set_ylabel("Albedo Difference: \n TARTES - PKM")
        axs5.axes.get_yaxis().set_visible(False)
        
    #### Subplot 5 sza
        
    axs7.plot(szalist, label = "sza",linewidth = lw,color='black',alpha = 0.8)
    axs7.set_ylabel("SZA [%s]"%(degree_sign))
    axs7.set_yticks([45,60,75,90])   
    axs7.text(-9, 75, '(e)', fontsize=12, weight = 'bold',backgroundcolor='w')
    
    if pkm != 1:
        axs6.plot(szalist, label = "sza",linewidth = lw,color='black',alpha = 0.8)
        axs6.set_ylabel("SZA [%s]"%(degree_sign))
        axs6.set_yticks([45,60,75,90])
        axs5.axes.get_yaxis().set_visible(False)
    
    #### Subplot 5 cf
    
    axs9.plot(cflist, label = "Cloud Cover",linewidth = 1.2,color='black',alpha = 0.8)
    
    axs9.set_ylim(-0.08,1.08)
    axs9.set_ylabel("Cloud Cover")
    axs9.text(-9, 0.75, '(f)', fontsize=12, weight = 'bold',backgroundcolor='w')
    axs8.axes.get_yaxis().set_visible(False)
    
    if pkm != 1:
        axs6.plot(cflist, label = "Cloud Cover",linewidth = 1.2,color='black',alpha = 0.8)
    
        axs7.set_ylim(-0.01,1.01)
        axs7.set_ylabel("Cloud Cover")
      
    #### Subplot 6 t2m
      
    ###     calculating the melt events using the melt water flux
      
    L1 = []
    L2 = [] 
    for i in range(0,len(meltflux_time)):
        if i == (i+1):
            L1.append(meltflux_time[i])
        else:
            L1.append(meltflux_time[i])
            L2.append(L1)
            L1 = []
    L2 = np.array(L2)
    meltevents = L2 > 10**-5        # We say that we have a melt event if the melt flux is higher than 10**-5. This way we have filtered the noise
    meltevents = meltevents * 1
    
    meltevents2 = np.sum(meltevents,axis=1)     # Now we say that if during any moment of the day we have melt, so not only at 1500 UTC, that the entire day is a melt event 
    meltevents3 = np.where(meltevents2 > 0)[0]      
    meltdiff = np.diff(meltevents3)
    melt_times = np.split(meltevents3, np.where(meltdiff > 1)[0]+2)     # Split into melt periods and extend by 1 (because it also lasts during the entire last day of the event, not only the start of the last day).

    minT = np.min(t2mlist-273.16)        # minimal t2m
    maxT = np.max(t2mlist-273.16)        # maximal t2m
    for i in range(0,len(melt_times)):
        axs10.add_patch(patches.Rectangle((melt_times[i][0], np.round(minT-10)),len(melt_times[i]),np.round(maxT + 10)- np.round(minT-10),color = "crimson",alpha = 0.8))      # remove background    
    
    ##
        
    axs10.plot(t2mlist -273.16, label = "t2m",linewidth = lw,color='black',alpha = 0.8)
    axs10.plot(nulline, linewidth = lw, color = "black", alpha = 0.8, ls = "--")
    axs10.set_yticks([0,-20,-40])
    axs10.set_ylim(np.min(t2mlist -273.16) - 2, np.max(t2mlist -273.16) + 2)
    axs10.set_ylabel("2-m Tempe- \n rature [%sC]"%(degree_sign))
    axs10.text(-9, -11, '(g)', fontsize=12, weight = 'bold',backgroundcolor='w')
    
    if pkm != 1:
        axs9.plot(t2mlist -273.16, label = "t2m",linewidth = lw, color='black',alpha = 0.8)
        axs9.plot(nulline, linewidth = lw, color = "black", alpha = 0.8, ls = "--")
        axs9.set_yticks([0,-20,-40])
        axs9.set_ylim(np.min(t2mlist -273.16) - 2, np.max(t2mlist -273.16) + 2)
        axs9.set_ylabel("2-m Tempe- \n rature [%sC]"%(degree_sign))
        axs8.axes.get_yaxis().set_visible(False)

    #### SSA AND FRESH SNOW LAYER PART

    depth = np.cumsum(thickness_time2,axis=0)  ## Calculating the depth
    
    timelist = np.zeros_like(depth)
    for i in range(0,len(xdays)):
        for j in range(0,len(ssa_time2)):
            timelist[j,i] = i
            
    xline = np.where(thickness_time2[0,:] == 0.0)[0]     # Making the list of where to draw a line where fresh snow is absent
    diffxline = np.diff(xline)
    splitlist = np.where(diffxline != 1)[0]
    splitlist2 = np.split(xline,splitlist+1)
    nofreshsnowlist = []
    for i in range(0,len(splitlist2)):
        nofreshsnowlist.append(np.append(splitlist2[i],splitlist2[i][-1]+1))
    
        
    depth2 = []    
    for i in range(0,len(ssa_time2[0,:])):
        list1 = list(depth[:,i])
        list1.insert(0,0.0)
        depth2.append(list1)
    depth2 = np.transpose(np.array(depth2))

    plot1 = axs12.pcolormesh(timelist,depth2[:-1],ssa_time2,cmap = "rainbow")
    for i in range(0,len(nofreshsnowlist)):
        axs12.plot(nofreshsnowlist[i],np.zeros_like(nofreshsnowlist[i])-0.015,color = 'k',linewidth= 2.5)
    axs12.set_yticks([0.0,0.1,0.2,0.3,0.4,0.5])
    axs12.set_ylim(-0.03,0.5)
    axs12.set_ylabel("Depth [m]")
    axs12.text(-9, 0.44, '(h)', fontsize=12, weight = 'bold',backgroundcolor='w')
    plt.xticks(xlist,monthlistabb)
    cbaxes = fig.add_axes([0.135, 0.07, 0.01, 0.12])  # This is the position for the colorbar
    axs11.get_yaxis().set_visible(False)
    
    
    ####
    # MAKING AND SAVING THE ACTUAL PLOT
    ####
    
    plt.tight_layout()
    plt.setp(axs.get_xticklabels(), visible=False)
    plt.subplots_adjust(hspace=.0)
    plt.setp(axs2.get_yticklabels(), visible=False)
    axs.set_xlim(xlist[0]-15,xlist[-1]+15)
    plt.xticks(xlist,monthlistabb)
    cbar = plt.colorbar(plot1,cax=cbaxes,ticks=[5,20,40,60],label = "SSA [m$^2$ kg$^{-1}$]",ticklocation='left')
    fig.text(0.5, 0.007, 'Time', ha='center', va='center')
    plt.savefig("%s/Time_series_%s_GMT_%s.png"%(savedir,timegmt,year))
    plt.savefig("%s/Time_series_%s_GMT_%s.pdf"%(savedir,timegmt,year))
    
    ####
    
    
